@@ -48,24 +48,3 @@ amount = len(funcs) * iters
 print('total method calls: %d' % amount)
 avg_pyro_msec = 1000.0 * duration / amount
 print('avg. time per method call: %.3f msec (%d/sec) (serializer: %s)' % (avg_pyro_msec, amount / duration, Pyro4.config.SERIALIZER))
-
-print('-------- BENCHMARK LOCAL OBJECT ---------')
-obj = bench.bench()
-begin = time.time()
-iters = 300000
-print("warmup...")
-for _ in range(iters):
-    funcs[0]()
-for i, f in enumerate(funcs, start=1):
-    print("call #%d, %d times... " % (i, iters), end="")
-    before = time.time()
-    for _ in range(iters):
-        f()
-    print("%.3f" % (time.time() - before))
-duration = time.time() - begin
-print('total time %.3f seconds' % duration)
-amount = len(funcs) * iters
-print('total method calls: %d' % amount)
-avg_normal_msec = 1000.0 * duration / amount
-print('avg. time per method call: %.3f msec (%d/sec)' % (avg_normal_msec, amount / duration // 1000 * 1000))
-print('Normal method call is %.0f times faster than Pyro method call.' % (avg_pyro_msec / avg_normal_msec))
