@@ -11,6 +11,7 @@ import os
 import sys
 import time
 import uuid
+import socket
 import unittest
 import warnings
 import Pyro4.core
@@ -343,6 +344,23 @@ class CoreTests(unittest.TestCase):
         self.assertFalse(Pyro4.core.URI.isUnixsockLocation("./x:name"))
         self.assertFalse(Pyro4.core.URI.isUnixsockLocation("foobar"))
 
+    def testProxyConnectedSocket(self):
+        s1, s2 = socket.socketpair()
+        s1.settimeout(0.1)
+        s2.settimeout(0.1)
+        try:
+            Pyro4.config.METADATA = False
+            with self.assertRaises(Pyro4.errors.PyroError):
+                Pyro4.Proxy("test")
+            with Pyro4.Proxy("test", connected_socket=s1) as p:
+                self.assertEquals("<<connected-socket>>:0", p._pyroUri.location)
+                self.assertIsNotNone(p._pyroConnection)
+                p._pyroRelease()
+                self.assertIsNotNone(p._pyroConnection)
+            self.assertIsNotNone(p._pyroConnection)
+        finally:
+            Pyro4.config.METADATA = True
+
     def testProxyCopy(self):
         u = Pyro4.core.URI("PYRO:12345@hostname:9999")
         p1 = Pyro4.core.Proxy(u)
@@ -498,6 +516,7 @@ class CoreTests(unittest.TestCase):
     def testProxyWithStmt(self):
         class ConnectionMock(object):
             closeCalled = False
+            keep_open = False
 
             def close(self):
                 self.closeCalled = True
@@ -535,8 +554,9 @@ class CoreTests(unittest.TestCase):
 
     def testProxyHmac(self):
         class ConnectionMock(object):
-            def __int__(self):
+            def __init__(self):
                 self.msgbytes = None
+                self.keep_open = False
             def close(self):
                 pass
             def send(self, msgbytes):
@@ -573,6 +593,7 @@ class CoreTests(unittest.TestCase):
         class ConnectionMock(object):
             def __init__(self):
                 self.timeout = config.COMMTIMEOUT
+                self.keep_open = False
 
             def close(self):
                 pass
@@ -777,7 +798,7 @@ class RemoteMethodTests(unittest.TestCase):
         def __exit__(self, *args):
             pass
 
-        def _pyroAsync(self, async=True):
+        def _pyroAsync(self, asynchronous=True):
             return self
 
         def __getattr__(self, item):
@@ -850,17 +871,17 @@ class RemoteMethodTests(unittest.TestCase):
         results = batch(oneway=True)
         self.assertIsNone(results)  # oneway always returns None
         self.assertEqual(4, len(proxy.result))  # should have done 4 calls, not 5
-        self.assertRaises(Pyro4.errors.PyroError, batch, oneway=True, async=True)  # oneway+async=booboo
+        self.assertRaises(Pyro4.errors.PyroError, batch, oneway=True, asynchronous=True)  # oneway+asynchronous=booboo
 
     def testBatchMethodAsync(self):
         proxy = self.BatchProxyMock()
         batch = Pyro4.core.batch(proxy)
         self.assertIsNone(batch.foo(42))
         self.assertIsNone(batch.bar("abc"))
-        self.assertIsNone(batch.pause(0.5))  # pause shouldn't matter with async
+        self.assertIsNone(batch.pause(0.5))  # pause shouldn't matter with asynchronous
         self.assertIsNone(batch.baz(42, "abc", arg=999))
         begin = time.time()
-        asyncresult = batch(async=True)
+        asyncresult = batch(asynchronous=True)
         duration = time.time() - begin
         self.assertLess(duration, 0.2, "batch oneway with pause should still return almost immediately")
         results = asyncresult.value
@@ -900,10 +921,10 @@ class RemoteMethodTests(unittest.TestCase):
             self.assertIsNone(proxy._pyroAsync(False))
             m = proxy.foo
             self.assertIsInstance(m, Pyro4.core._RemoteMethod)
-            self.assertIsNone(Pyro4.async(proxy))
+            self.assertIsNone(Pyro4.asyncproxy(proxy))
             m = proxy.foo
             self.assertIsInstance(m, Pyro4.core._AsyncRemoteMethod)
-            self.assertIsNone(Pyro4.async(proxy, False))
+            self.assertIsNone(Pyro4.asyncproxy(proxy, False))
             m = proxy.foo
             self.assertIsInstance(m, Pyro4.core._RemoteMethod)
         finally:
@@ -911,7 +932,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testAsyncMethod(self):
         proxy = self.AsyncProxyMock()
-        Pyro4.core.async(proxy)
+        Pyro4.core.asyncproxy(proxy)
         begin = time.time()
         result = proxy.pause_and_divide(0.2, 10, 2)  # returns immediately
         duration = time.time() - begin
@@ -930,7 +951,7 @@ class RemoteMethodTests(unittest.TestCase):
                 return value + amount
 
         proxy = self.AsyncProxyMock()
-        Pyro4.core.async(proxy)
+        Pyro4.core.asyncproxy(proxy)
         result = proxy.pause_and_divide(0.2, 10, 2)  # returns immediately
         holder = AsyncFunctionHolder()
         result.then(holder.asyncFunction, amount=2) \
@@ -949,7 +970,7 @@ class RemoteMethodTests(unittest.TestCase):
             return 1 // 0  # crash
 
         proxy = self.AsyncProxyMock()
-        Pyro4.core.async(proxy)
+        Pyro4.core.asyncproxy(proxy)
         result = proxy.pause_and_divide(0.2, 10, 2)  # returns immediately
         result.then(crashingAsyncFunction).then(normalAsyncFunction, 2)
         try:
@@ -961,7 +982,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testAsyncMethodTimeout(self):
         proxy = self.AsyncProxyMock()
-        Pyro4.core.async(proxy)
+        Pyro4.core.asyncproxy(proxy)
         result = proxy.pause_and_divide(1, 10, 2)  # returns immediately
         self.assertFalse(result.ready)
         self.assertFalse(result.wait(0.5))  # won't be ready after 0.5 sec
